﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CoreLibrary.Datos.Modelo;
namespace CoreLibrary.Controlador.Filtros
{
    /// <summary>
    /// Clase que tiene una lista interna de elementos de la forma string nombre int cantidad
    /// Internamente implementa una lista con todos los de su clase.
    /// 
    /// </summary>
    public class ElementoOcurrencia : IComparable
    {
        public const string SEPARADOR_IZQUIERDA = "[";
        public const string SEPARADOR_DERECHA = "]";

        public int CompareTo(Object parametro)
        {
            ElementoOcurrencia a = (ElementoOcurrencia)parametro;
            if (a.Contador > this.Contador)
                return 1;
            if (a.Contador == this.Contador)
                return this.Nombre.CompareTo(a.Nombre);
            else
                return -1;
        }




        public ElementoOcurrencia(string elemento)
        {
            this._nombre = elemento;
            _contador = 1;
        }
        public ElementoOcurrencia(string elemento,int contador)
        {
            this._nombre = elemento;
            _contador = contador;
        }
        public void incrementarContador()
        {
            _contador++;
        }
        private int _contador;
        private string _nombre;
       
        public int Contador
        {
            set
            {
                this._contador = value;
            }
            get
            {
                return _contador;
            }
        }

        public string Nombre
        {
            set
            {
                this._nombre = value;

            }
            get
            {
                return _nombre;
            }
        }

        public override string ToString()
        {
            return this.Nombre + SEPARADOR_IZQUIERDA+ this._contador.ToString() + SEPARADOR_DERECHA;
        }

        /// <summary>
        /// Se le pasa la lista de ElementoOcurrencia por parametro, si existe el elemento lo incrementa internamente si no, 
        /// se agrega a la lista.
        /// Si el elemento es una cadena vacia (no se ha inicializado) no lo contabiliza
        /// </summary>
        /// <param name="eos"></param>
        public void incrementarAñadirLista(List<ElementoOcurrencia> eos)
        {
             ElementoOcurrencia eo = eos.FirstOrDefault(p => p.Nombre.Equals(this.Nombre));
            
            if (eo != null)
                eo.incrementarContador();
            else if(this._nombre !=null && !this.Nombre.Equals(string.Empty))
                eos.Add(this);

         }

        /// <summary>
        /// Convierte un elemento ocurrencia a partir de una cadena que incluye los corchetes
        /// </summary>
        /// <param name="elemento"></param>
        /// <returns></returns>
        public static ElementoOcurrencia fromStringEntero(string elemento)
        {
            string[] valores = elemento.Split(SEPARADOR_IZQUIERDA.ToCharArray());
            string nombre = valores[0];
            int numero = int.Parse(valores[1].Remove(valores[1].Length-1));
            return new ElementoOcurrencia(nombre,numero);
          


        }
        /// <summary>
        /// Convierte un elemento ocurrencia a partir de una cadena que no incluye los corchetes
        /// </summary>
        /// <param name="elemento"></param>
        /// <returns></returns>
        public static ElementoOcurrencia fromString(string elemento)
        {
            
           return new ElementoOcurrencia(elemento);

          

        }


    }
}
