﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CoreLibrary.Datos.Modelo;
using CoreLibrary.Datos;
using CoreLibrary.Datos.Modelo;
using CoreLibrary.Literales;
namespace CoreLibrary.Controlador.Tipo
{
    /// <summary>
    /// Esta clase sirve para almacenar un BIT (Bool, int y Text). 
    /// Internamente funciona combinando las funcionalidades de TBI (Tiene propiedad booleana y entera) y de TTEXT (Posee un idexador para acceder al texto de cada idioma)
    /// La forma en la que se almacena será: {Bool}|{Int}|{Text} por ejemplo: True|345|Ejemplo de texto.
    /// </summary>
   
    public  class TBIT : TBI
    {
      List<ValoresEP> vcs;
       /// <summary>
        /// Crea una nueva instancia del tipo TBIT
        /// </summary>
        /// <param name="ec"></param>
        /// <param name="pc"></param>
        /// <param name="valor"></param>
        public  TBIT(Elemento ec, Propiedad pc,bool valorBool,int valorInt, string valorString,string cod)
        {
            this.nuevo(ec, pc, valorBool, valorInt, valorString, cod);
        }

        public TBIT(ValoresEP vc)
        {
            this.cargar(vc);
        }
        protected void nuevo(Elemento ec, Propiedad pc, bool valorBool, int valorInt, string valorString, string cod)
        {
            Idioma idioma = GestionDatos.Instance.devolverIdiomaPorCod(cod);
            string valor = valorBool.ToString() + LTipoPrimitivo.Separador + valorInt.ToString() + LTipoPrimitivo.Separador + valorString;
            ValoresEP vc = new ValoresEP(ec, pc, valor, idioma);
            this.vcs = new List<ValoresEP>();
            this.vcs.Add(vc);
            this.cargar(ref vcs);
         
        }
        protected void cargar(ValoresEP vc)
        {
            this.vcs = new List<ValoresEP>();
            this.vcs.Add(vc);
            this.cargar(ref vcs);
            
        }
     
        /// <summary>
        /// Indexador, para obtener un valor pasar el codigo de idioma por indice.
        /// Para agregar un nuevo valor asignar o modificar uno existente pasar el codigo de idioma por indice., automaticamente se ejecutaran
        /// las operaciones pertinentes y se almacenara en la base de datos.
        ///
        /// </summary>
        /// <param name="cod"></param>
        /// <returns></returns>
        public string this[string cod]
        {
            get
            {
                 Idioma idioma = GestionDatos.Instance.devolverIdiomaPorCod(cod);
                ValoresEP vc = this.vcs.FirstOrDefault(v => v.Idioma.Id == idioma.Id);
                if (vc == null)
                    return string.Empty;
                return vc.Valor.Split(LTipoPrimitivo.Separador)[2];
            }
            set
            {
              

                Idioma idioma = GestionDatos.Instance.devolverIdiomaPorCod(cod);
                ValoresEP vc = this.vcs.FirstOrDefault(v => v.Idioma.Id == idioma.Id);
                //Si ya contiene la propiedad la modifica
                if (vc != null)
                {
                    this.guardarValor(ref this.vcs, value,cod);
                }
                //Si no, crea una nueva y la añade
                else
                {
                    string valor = this.ValorBooleano.ToString() + LTipoPrimitivo.Separador + this.ValorEntero.ToString() + LTipoPrimitivo.Separador + value;
                    vc = new ValoresEP(this.Elemento, this.Propiedad, valor, idioma);
                    this.vcs.Add(vc);
                }
            }
        }

    }
}
