﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CoreLibrary.Controlador;
using CoreLibrary.Datos.Modelo;
using CoreLibrary.Literales;
namespace CoreLibrary.Controlador.Tipo
{
     
    public abstract class TipoBase:ControladorBase
    {
        private Elemento _Elemento;

        public Elemento Elemento
        {
            get { return this.ValoresEP.Elemento; }
           
        }

        public string Nombre
        {
            get
            {
               return this.Propiedad.Nombre;
            }
        }
        public bool Baja
        {
            get
            {
                return this.Propiedad.Baja;
            }
        }

        public Propiedad Propiedad
        {
            get { return this.ValoresEP.Propiedad; }
           
        }
        private ValoresEP _valoresEP;

        public ValoresEP ValoresEP
        {
            get { return _valoresEP; } //TODO Cuando se refactorice ver si hay forma de que desaparezca.
            set
            {
                this._valoresEP = value;
            }
        }

        #region Consultas
        /// <summary>
        /// Devuelve el nombre de la propiedad para el idioma pasado por parametro.
        /// </summary>
        /// <param name="cod"></param>
        /// <returns></returns>
        public string consultarNombreTraducido(string cod)
        {
            return this.Propiedad.devolverTraduccionPropiedad(cod);
        }
        /// <summary>
        /// Carga una instancia de controlador base a traves de un valor controlador asociado.
        /// </summary>
        /// <param name="vc"></param>
        protected void cargar(ValoresEP vc)
        {
            if (vc != null)
            {
                this.ValoresEP = vc;
            }
            
        }
        //Carga una instancia de controlador base a traves de una propiedad y elemento asociado, en caso de que no la encuentre el vc es null.
        protected void cargar(Elemento ec, Propiedad pc)
        {
            List<ValoresEP> vcs = (from v in ec.ValoresEP
                                          where v.Nombre.Equals(pc.Nombre)
                                          select v).ToList();
            //Si no la encuentra es false. Y la entidad será null.
            if (vcs.Count == 0)
                return;
           
            if (vcs.Count == 1)
                cargar(vcs[0]);
            else if (vcs.Count > 1)
                cargar(ref vcs);
           

        }

        /// <summary>
        /// Se utiliza para cargar textos
        /// </summary>
        /// <param name="vcs"></param>
        protected void cargar(ref List<ValoresEP> vcs)
        {
            
            ValoresEP vc = vcs.FirstOrDefault();
            
            vcs = vc.Elemento.ValoresEP.Where(v => v.Propiedad.Id == vc.Propiedad.Id).ToList();
            this._valoresEP = vcs.FirstOrDefault();
           

        }
        /// <summary>
        /// Se almacena el valor en la base de datos, se indica la posicion (como si fuera un array) y el valor a  almacenar.
        /// </summary>
        /// <param name="posicion"></param>
        /// <param name="value"></param>
        protected void guardarValor(int posicion, string value)
        {

            string[] valores = this._valoresEP.Valor.Split(LTipoPrimitivo.Separador);
            //se guarda el tipo como una cadena (en la  posicion psasada por parametro)
            valores[posicion] = value;
            //Se asignan los cambios
            _valoresEP.Valor = string.Empty;
            foreach (string v in valores)
                _valoresEP.Valor += v + LTipoPrimitivo.Separador;
            //Se elimina el ultimo caracter separador
           this._valoresEP.Valor= _valoresEP.Valor.Remove(_valoresEP.Valor.Length - 1);
            _valoresEP.guardarVEP();



        }
        /// <summary>
        /// Metodo que se utiliza para almacenar el texto de las cadenas, teniendo en cuenta que el valor del texto siempre se almacenara en la ultima posicion
        /// 
        /// </summary>
        /// <param name="posicion"></param>
        /// <param name="value"></param>
        /// <param name="cod"></param>
        protected void guardarValor(ref  List<ValoresEP> vcs, string value, string cod)
        {

            string[] valores = this._valoresEP.Valor.Split(LTipoPrimitivo.Separador);

            foreach (ValoresEP vc in vcs)
            {
               //Se guarda el valor del texto para el idioma si es distinto del idioma que se esta modificando
                string bufferIdioma = string.Empty;
                if (vc.Idioma.Cod != cod)
                    bufferIdioma = valores[valores.Length - 1];
                //se guarda el tipo como una cadena (en la  posicion psasada por parametro)
                valores[valores.Length - 1] = value;
               
                //Se asignan los cambios           
                vc.Valor = string.Empty;
                for (int i = 0; i < valores.Length; i++)
                {
                    //Si esta modificando un tipo distinto del de texto, o si esta modificando el de texto para el idioma actual
                    //entra
                    if (i != valores.Length - 1 || vc.Idioma.Cod == cod)
                        vc.Valor += valores[i] + LTipoPrimitivo.Separador;
                    else
                        vc.Valor += bufferIdioma + LTipoPrimitivo.Separador;
                }
                //Se elimina el ultimo caracter separador
                   vc.Valor=  vc.Valor.Remove(vc.Valor.Length - 1);
                vc.guardarVEP();

            }
            this._valoresEP = vcs.FirstOrDefault();
        }



        /// <summary>
        /// Devuelve el valor para la posicion actual, el que reciba el string se debe encargar de pasarlo al tipo adecuado.
        /// </summary>
        /// <param name="posicion"></param>
        /// <returns></returns>
        protected string consultarValor(int posicion)
        {
            return this._valoresEP.Valor.Split(LTipoPrimitivo.Separador)[posicion];

        }
        #endregion


   
       
    }
}
