﻿using System;
using System.Net;

using System.Xml.Serialization;
using System.IO;
using CoreLibrary.Datos;
using System.Collections.Generic;
using CoreLibrary.Datos.Modelo;
using System.Linq;
#if WINDOWS_PHONE
using System.IO.IsolatedStorage;
#endif

namespace CoreLibrary.Datos.Almacen
{
    public class DatosXML : IModeloReader
    {
        #region Vars
        private string _cadConexion;
        private string ELEMENTOS, CATEGORIA, DIRECCION, IDIOMA, PROPIEDAD, TRADUCCIONPROPIEDAD, VALORESEP, CATEGORIAPROPIEDAD;
        #endregion
        public DatosXML(string cadena)
        {
            this._cadConexion = "\\";
            ELEMENTOS = (_cadConexion + "Elementos");
            CATEGORIA = (_cadConexion + "Categoria");
            CATEGORIAPROPIEDAD = (_cadConexion + "CategoriaPropiedad");
            DIRECCION = (_cadConexion + "Direccion");
            IDIOMA = (_cadConexion + "Idioma");
            PROPIEDAD = (_cadConexion + "Propiedad");
            TRADUCCIONPROPIEDAD = (_cadConexion + "TraduccionPropiedad");
            VALORESEP = (_cadConexion + "ValoresEP");

        }
        #region Consultas
        /// <summary>
        /// Devuelve una lsita de CBAse pasada por parametro.
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        private List<T> getCBases<T>(string path) where T : ControladorBase
        {
            List<T> cBases = cargarDesdeUnFichero<T>(path);
            if (cBases == null)
            {
                cBases = new List<T>();

            }
            return cBases;
        }
        #endregion
        #region Update

        private void modificarCbase<T>(T cbase, string path) where T : ControladorBase
        {
            List<T> cBases = cargarDesdeUnFichero<T>(path);

            T cb=cBases.First(c => c.Id == cbase.Id);
            cBases[cBases.IndexOf(cb)] = cbase;
            listaAFichero<T>(cBases, path);
            


        }

        private void modificarCbaseList<T>(List<T> cBasesU, string path) where T : ControladorBase
        {
            List<T> cBases = cargarDesdeUnFichero<T>(path);

            foreach (var item in cBasesU)           
                cBases[cBases.IndexOf(item)] = item;

            listaAFichero<T>(cBases, path);

        }

        #endregion
        #region Insertar
        /// <summary>
        /// Inserta una clase derivada de cBase en la ruta especificacda.
        /// </summary>
        /// <param name="cBase"></param>
        /// <param name="path"></param>
        /// <param name="cBases"></param>
        private void insertarCBase<T>(T cBase, string path, List<T> cBases = null) where T : ControladorBase
        {

            if (cBases == null)
                cBases = cargarDesdeUnFichero<T>(path);

            if (cBases == null)
                cBases = new List<T>();


            cBases.Add(cBase);
            listaAFichero<T>(cBases, path);

        }
        /// <summary>
        /// Inserta una lista de CbasesL.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="cBaseN"></param>
        /// <param name="path"></param>
        /// <param name="cBases"></param>
        private void insertarCBaseList<T>(List<T> cBaseN, string path, List<T> cBases = null) where T : ControladorBase
        {

            if (cBases == null)
                cBases = cargarDesdeUnFichero<T>(path);

            if (cBases == null)
                cBases = new List<T>();

            foreach (var item in cBaseN)
            {
              
                cBases.Add(item);
                
            }
            listaAFichero<T>(cBases, path);

        }




        #endregion


        #region Interaccion con fichero
        /// <summary>
        /// Convierte una lista de controlador base en un fichero
        /// </summary>
        /// <param name="objeto"></param>
        /// <returns></returns>
        public void listaAFichero<T>(List<T> lista, string path) where T : ControladorBase
        {
#if WINDOWS_PHONE
            IsolatedStorageFile userFile = IsolatedStorageFile.GetUserStoreForApplication();
            if(userFile.FileExists(path))
                userFile.DeleteFile(path);

            using(IsolatedStorageFileStream fileStream = userFile.OpenFile(path, System.IO.FileMode.CreateNew))
            {

                XmlSerializer serializador = new XmlSerializer(typeof(List<T>));
                serializador.Serialize(fileStream, lista);

            }
			
			
#elif MONO
			
            if (lista == null )
                return;
            if (lista.Count == 0)
                return;
            if (File.Exists(path))
                File.Delete(path);
            using(FileStream fileStream = File.Open(path, System.IO.FileMode.CreateNew))
           {

               XmlSerializer serializador = new XmlSerializer(typeof(List<T>));
               serializador.Serialize(fileStream, lista);

           }
			
			
#endif
        }

        /// <summary>
        /// Convierte una lista de controlador base en un fichero
        /// </summary>
        /// <param name="objeto"></param>
        /// <returns></returns>
        //public void listaAFichero<T>(List<T> lista, string path) where T : ControladorBase
        //{

          

        //    if(File.Exists(path))
        //        File.Delete(path);


        //    using(FileStream fileStream = File.Open(path, System.IO.FileMode.CreateNew))
        //    {

        //        XmlSerializer serializador = new XmlSerializer(typeof(List<T>));
        //        serializador.Serialize(fileStream, lista);

        //    }

        //}



        /// <summary>
        /// Carga desde un fichero la lista. Si el fichero no existe retorna null
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public List<T> cargarDesdeUnFichero<T>(string path) where T : ControladorBase
        {
#if WINDOWS_PHONE
			
            IsolatedStorageFile userFile = IsolatedStorageFile.GetUserStoreForApplication();
            if (!userFile.FileExists(path))
                return null;


            List<T> lista = new List<T>();
            using (IsolatedStorageFileStream fileStream = userFile.OpenFile(path, System.IO.FileMode.Open))
            {

                XmlSerializer serializador = new XmlSerializer(typeof(List<T>));
               lista=(List<T>) serializador.Deserialize(fileStream);
            //Se establece al State Unchanged
               foreach (var item in lista)
                  item.State = State.Unchanged;
               
            }


              return lista;

          
#elif MONO


            List<T> lista = null;

			
			if(!File.Exists(path)  )
				return null;
            if((new FileInfo(path).Length ==0))
                return null;
			
			
			using (FileStream fileStream = File.Open(path,FileMode.Open))
			{
			XmlSerializer serializador = new XmlSerializer(typeof(List<T>));
			lista=(List<T>) serializador.Deserialize(fileStream);
			 foreach (var item in lista)
                  item.State = State.Unchanged;
			}
			
			
			  return lista;
#endif
            return null;

        }
        ///// <summary>
        ///// Carga desde un fichero la lista. Si el fichero no existe retorna null
        ///// </summary>
        ///// <param name="path"></param>
        ///// <returns></returns>
        //public List<T> cargarDesdeUnFichero<T>(string path) where T : ControladorBase
        //{
        //    if(!File.Exists(path))
        //        return null;


        //    List<T> lista = new List<T>();
        //    using(FileStream fileStream = File.Open(path, System.IO.FileMode.Open))
        //    {

        //        XmlSerializer serializador = new XmlSerializer(typeof(List<T>));
        //        lista = (List<T>)serializador.Deserialize(fileStream);

        //    }




        //    return lista;
        //}
        #endregion


        #region Implementación de la interfaz
        List<Elemento> IModeloReader.cargarElemento()
        {

            return this.getCBases<Elemento>(ELEMENTOS);
        }

        List<Propiedad> IModeloReader.cargarPropiedades()
        {
            return this.getCBases<Propiedad>(PROPIEDAD);
        }

        List<Categoria> IModeloReader.cargarCategoria()
        {
            return this.getCBases<Categoria>(CATEGORIA);
        }

        List<ValoresEP> IModeloReader.cargarValoresEP()
        {
            return this.getCBases<ValoresEP>(VALORESEP);
        }

        List<Idioma> IModeloReader.cargarIdioma()
        {
            return this.getCBases<Idioma>(IDIOMA);
        }

        List<CategoriaPropiedad> IModeloReader.cargarCategoriaPropiedad()
        {
            return this.getCBases<CategoriaPropiedad>(CATEGORIAPROPIEDAD);
        }

        List<TraduccionPropiedad> IModeloReader.cargarTraduccionPropiedad()
        {
            return this.getCBases<TraduccionPropiedad>(TRADUCCIONPROPIEDAD);
        }

        List<Direccion> IModeloReader.cargarDireccion()
        {
            return this.getCBases<Direccion>(DIRECCION);
        }


        void IModeloReader.nuevoIdioma(Idioma i)
        {
            this.insertarCBase<Idioma>(i, IDIOMA);
        }


        void IModeloReader.nuevoElemento(Elemento elemento)
        {
            Elemento e = new Elemento();
            e.Nombre = elemento.Nombre;
            e.Baja = elemento.Baja;
            e.Id = elemento.Id;


            this.insertarCBase<Elemento>(e, ELEMENTOS);
        }

        void IModeloReader.nuevaPropiedad(Propiedad propiedad)
        {
            this.insertarCBase<Propiedad>(propiedad, PROPIEDAD);
        }

        void IModeloReader.nuevoCategoria(Categoria categoria)
        {
            this.insertarCBase<Categoria>(categoria, CATEGORIA);
        }

        void IModeloReader.nuevaCategoriaPropiedad(CategoriaPropiedad categoriaPropiedad)
        {
            this.insertarCBase<CategoriaPropiedad>(categoriaPropiedad, CATEGORIAPROPIEDAD);
        }

        //void IModeloReader.nuevoDireccion(Direccion direccion)
        //{
        //    this.insertarCBase<Direccion>(direccion, DIRECCION);
        //}

        void IModeloReader.nuevaTraduccionPropiedad(TraduccionPropiedad traduccionPropiedad)
        {
            this.insertarCBase<TraduccionPropiedad>(traduccionPropiedad, TRADUCCIONPROPIEDAD);
        }

        void IModeloReader.nuevaValoresEP(ValoresEP vep)
        {
            this.insertarCBase<ValoresEP>(vep, VALORESEP);
        }

        void IModeloReader.actualizarElemento(Elemento elemento)
        {
            this.modificarCbase<Elemento>(elemento, ELEMENTOS);
        }

        void IModeloReader.actualizarPropiedad(Propiedad propiedad)
        {
            this.modificarCbase<Propiedad>(propiedad, PROPIEDAD);
        }

        void IModeloReader.actualizarCategoria(Categoria categoria)
        {
            this.modificarCbase<Categoria>(categoria, CATEGORIA);
        }

        //void IModeloReader.actualizarDireccion(Direccion direccion)
        //{
        //    this.modificarCbase<Direccion>(direccion, DIRECCION);
        //}

        void IModeloReader.actualizarTraduccionPropiedad(TraduccionPropiedad traduccionPropiedad)
        {
            this.modificarCbase<TraduccionPropiedad>(traduccionPropiedad, TRADUCCIONPROPIEDAD);
        }

        void IModeloReader.actualizarValoresEP(ValoresEP vep)
        {
            this.modificarCbase<ValoresEP>(vep, VALORESEP);
        }
        #endregion






        void IModeloReader.nuevoListaElemento(List<Elemento> elementos)
        {
            List<Elemento> el = new List<Elemento>();
            foreach (var elemento in elementos)
            {
                Elemento e = new Elemento();
                e.Nombre = elemento.Nombre;
                e.Baja = elemento.Baja;
                e.Id = elemento.Id;
                el.Add(e);
            }
           

            this.insertarCBaseList<Elemento>(el, ELEMENTOS);
        }

        void IModeloReader.nuevaListaPropiedad(List<Propiedad> propiedades)
        {
            this.insertarCBaseList<Propiedad>(propiedades, PROPIEDAD);
        }

        void IModeloReader.nuevoListaCategoria(List<Categoria> categorias)
        {
            this.insertarCBaseList<Categoria>(categorias, CATEGORIA);
        }

        void IModeloReader.nuevaListaCategoriaPropiedad(List<CategoriaPropiedad> categoriaPropiedades)
        {
            this.insertarCBaseList<CategoriaPropiedad>(categoriaPropiedades, CATEGORIAPROPIEDAD);
        }

        void IModeloReader.nuevaListaTraduccionPropiedad(List<TraduccionPropiedad> traduccionPropiedades)
        {
            this.insertarCBaseList<TraduccionPropiedad>(traduccionPropiedades, TRADUCCIONPROPIEDAD);
        }

        void IModeloReader.nuevaListaValoresEP(List<ValoresEP> vep)
        {
            this.insertarCBaseList<ValoresEP>(vep, VALORESEP);
        }

        void IModeloReader.actualizarListaElementos(List<Elemento> elementos)
        {
            this.modificarCbaseList<Elemento>(elementos, ELEMENTOS);
        }

        void IModeloReader.actualizarListaPropiedad(List<Propiedad> propiedades)
        {
            this.modificarCbaseList<Propiedad>(propiedades, PROPIEDAD);
        }

        void IModeloReader.actualizarListaCategoria(List<Categoria> categorias)
        {
            this.modificarCbaseList<Categoria>(categorias, CATEGORIA);
        }


        void IModeloReader.actualizarListaTraduccionPropiedad(List<TraduccionPropiedad> traduccionPropiedad)
        {
            this.modificarCbaseList<TraduccionPropiedad>(traduccionPropiedad, TRADUCCIONPROPIEDAD);
        }

        void IModeloReader.actualizarListaValoresEP(List<ValoresEP> vep)
        {
            this.modificarCbaseList<ValoresEP>(vep, VALORESEP);
        }
    }
}
