﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using CoreLibrary.Datos.Modelo;
using CoreLibrary.Controlador;
using CoreLibrary.Controlador.Colecciones;
using System.Threading;
namespace CoreLibrary.Datos
{
    /// <summary>
    /// CLase que aplica el patrón Singleton (sólo se instancia una vez) la clase controla los datos que se cargan de la base de datos.
    /// </summary>
   
    //LOS IDS DE LAS ENTIDADES QUE SE ASIGNAN EN LA BASE DE DATOS NO ESTAN DISPONIBLES EN A APLICACION
    //VER SI ES NECESARIO O NO.d
    public sealed class GestionDatos
    {
        #region Singleton
        static readonly GestionDatos instance = new GestionDatos();
        static GestionDatos()
        {
            
        }
        GestionDatos()
        {
            
                this.inicializar();
                this.cargar();
            

           
        }
        public static GestionDatos Instance
        {
            get
            {
                return instance;
            }
        }
        #endregion

        #region Variables
        private Dictionary<int, List<ValoresEP>> _tablaElementosValoresEP, _tablaPropiedadesValoresEP;

       

       
        private List<Idioma> __idiomas;
        private List<Elemento> __Elmentos;
        private List<Propiedad> __propiedades;
        private List<TraduccionPropiedad> __traducciones;
        private List<ValoresEP> __valores;
        private List<Categoria> __categoria;
        private List<CategoriaPropiedad> __categoriaPropiedad;
        private List<Direccion> __direcciones;
     
        private IModeloReader _iModelo;
        
       
        #endregion
        
        #region Propiedadees
        /// <summary>
        ///  Tabla de elementos con sus valoresEP correspondientes. Siendo la key el idelemento(int) y el valor una lista de valoresEP correspondiente a dicha key.
        /// </summary>
        public Dictionary<int,List<ValoresEP>> TablaElementosValoresEP  
        {
            get { return _tablaElementosValoresEP; }

        }
        /// <summary>
        ///  Tabla de propiedades con sus valoresEP correspondientes. Siendo la key el idpropiedad(int) y el valor una lista de valoresEP correspondiente a dicha key.
        /// </summary>
        public Dictionary<int, List<ValoresEP>> TablaPropiedadesValoresEP
        {
            get { return _tablaPropiedadesValoresEP; }

        }
        public Categoria this[string categoria]
        {
            get
            {
                return this.Categorias.FirstOrDefault(c => c.Nombre.Equals(categoria));
            }

        }

       

        public List<Direccion> Direcciones
        {
            get { return __direcciones; }
            set { __direcciones = value; }
        }
        public List<CategoriaPropiedad> CategoriaPropiedad
        {
            get
            {
                return this.__categoriaPropiedad;
            }


        }
        public List<Categoria> Categorias
        {
            get
            {
                return this.__categoria;
            }
            set
            {
                this.__categoria = value;

            }

        }
        public List<Idioma> Idiomas
        {
            get
            {
                return this.__idiomas;
            }

        }
        public List<Elemento> Elementos
        {
            get
            {
                return this.__Elmentos;
            }

        }
        public List<Propiedad> Propiedades
        {
            get
            {
                return this.__propiedades;
            }

        }
        public List<TraduccionPropiedad> Traducciones
        {
            get
            {
                return this.__traducciones;
            }

        }
        public List<ValoresEP> Valores
        {
            get
            {
                return this.__valores;
            }

        }

       
        #endregion
       
        #region MiembrosPrivados
        private void inicializar()
        {
            _iModelo = Config.OrigenDeDatos();
        }
        #endregion

        #region AccesoADatos
        
        public void cargar()
        {
                this.__idiomas = _iModelo.cargarIdioma();
                this.__Elmentos=_iModelo.cargarElemento();
                this.__propiedades= _iModelo.cargarPropiedades();
                this.__traducciones = _iModelo.cargarTraduccionPropiedad();             
                this.__valores = _iModelo.cargarValoresEP();
                this.__categoria=_iModelo.cargarCategoria();
                this.__categoriaPropiedad = _iModelo.cargarCategoriaPropiedad();
                this.__direcciones = _iModelo.cargarDireccion();

                //Mete los valoresEP de cada elemento en un Hash que para cada idElemento contiene una list
                this._tablaElementosValoresEP = new Dictionary<int, List<ValoresEP>>();
              
                foreach (var elemento in Elementos)     
                   _tablaElementosValoresEP.Add(elemento.Id,this.Valores.Where(v => v.IdElemento == elemento.Id).ToList());
                //Mete los valoresEP de cada propiedad en un Hash que para cada idElemento contiene una list
                this._tablaPropiedadesValoresEP =  new Dictionary<int, List<ValoresEP>>();
              
                foreach (var propiedad in Propiedades)
                    _tablaPropiedadesValoresEP.Add(propiedad.Id, this.Valores.Where(v => v.IdPropiedad == propiedad.Id).ToList());


               //this._tablaElementosValoresEP = _iModelo.cargarTablaDeValoresElementos(this.Elementos);
               //this._tablaPropiedadesValoresEP = _iModelo.cargarTablaDeValoresPropiedades(this.Propiedades);
        }
        

       
        #endregion
        #region ModificarDatos
       
        public void modificarElemento(Elemento e)
        {
            e.State = State.Modify;
            //Sustituye el objeto modificado en la lista. //TODO Comprobar que sea necesario la tarea es para el resto de entidades
            this.Elementos[this.Elementos.IndexOf(this.Elementos.FirstOrDefault(el => el.Id == e.Id))] = e;
       
        }
        public void modificarDireccion(Direccion dir)
        {
            
          // _iModelo.actualizarDireccion(dir);

            
        }
        public void modificarPropiedad(Propiedad p)
        {

            p.State = State.Modify;
            //Sustituye el objeto modificado en la lista.
            this.Propiedades[this.Propiedades.IndexOf(p)] = p;
           
        }

        public void modificarTraduccion(TraduccionPropiedad tp)
        {
                     tp.State = State.Modify;
                //Sustituye el objeto modificado en la lista.
                this.Traducciones[this.Traducciones.IndexOf(tp)] = tp;
        }

        public void modificarValor(ValoresEP vep)
        {
           
           //TODO Comprobar si realmente cambia en la lista por referencia o no
            //este metodo es muy costoso, ya que el numero de vep es muy elevado.


            vep.State = State.Modify;
                this.Valores[this.Valores.IndexOf(vep)] = vep;
            //Sustituye el objeto modificado en la lista. (El idioma si no existe es 0)          
                //this.Valores[this.Valores.FirstIndex(vepl => vepl._idPropiedad == vep._idPropiedad
                //   && vepl._idElemento == vep._idElemento && vepl._idIdioma==vep._idIdioma)] = vep;
              
            //Elementos
            List<ValoresEP> valores =(List<ValoresEP>) this.TablaElementosValoresEP[vep.IdElemento];

            valores[valores.IndexOf(vep)] = vep; //TODO Ver si hace falta (puede que no se asigne solo) lo mismo abajo
            this.TablaElementosValoresEP[vep.IdElemento] = valores;

           //Propiedades
            
            valores = (List<ValoresEP>)this.TablaPropiedadesValoresEP[vep.IdPropiedad];

            valores[valores.IndexOf(vep)] = vep; //TODO Ver si hace falta (puede que no se asigne solo) lo mismo abajo
            this.TablaPropiedadesValoresEP[vep.IdPropiedad] = valores;


          
        }
        #endregion
       
        #region GuardarDatos
        /// <summary>
        /// Se le pasa un elementoC (nuevo)por parametro y lo almacena en la base de datos, posteriormete actualiza todas las clases
        /// </summary>
        /// <param name="nombre"></param>
        public void guardarElemento(Elemento  e)
        {
                this.Elementos.Add(e);
                this.TablaElementosValoresEP.Add(e.Id, new List<ValoresEP>());
                e.State = State.New;
           

        }
        /// <summary>
        /// Se le pasa una propiedadC(nueva) por parametro y la almacena en la base de datos, posteriormete actualiza todas las clases
        /// </summary>
        /// <param name="p"></param>
        public void guardarPropiedad(Propiedad p)
        {

            this.Propiedades.Add(p);
            this.TablaPropiedadesValoresEP.Add(p.Id, new List<ValoresEP>());           
            p.State = State.New;
        }
        /// <summary>
        /// Se le pasa una traduccionpropiedad (nueva) por parametro y lo almacena en la base de datos, posteriormente actualiza todas las clases.
        /// </summary>
        /// <param name="tc"></param>
        public void guardarTraduccionPropiedad(TraduccionPropiedad tp)
        {              
           //     _iModelo.nuevaTraduccionPropiedad(tp);               
                this.Traducciones.Add(tp);
                tp.State = State.New;
        }
        /// <summary>
        /// Se le pasa un valor de una propiedad (nueva) independientemente del valor del que se trata lo almacena en la base de datos y posteriormente actualiza todas las clases.
        /// </summary>
        /// <param name="vc"></param>
        public void guardarValorPropiedad(ValoresEP vep)
        {
            
                 
             

                //Generales
                this.Valores.Add(vep);
               
                //Elementos
                List<ValoresEP> valores = (List<ValoresEP>)this.TablaElementosValoresEP[vep.IdElemento];
                valores.Add(vep);
                this.TablaElementosValoresEP[vep.IdElemento] = valores; //TODO Puede que no haga falta
                
                //propiedades
                valores = (List<ValoresEP>)this.TablaPropiedadesValoresEP[vep.IdPropiedad];
                valores.Add(vep);
                this.TablaPropiedadesValoresEP[vep.IdPropiedad] = valores;
                vep.State = State.New;
          
        }
        public void guardarCategoriaPropiedad(CategoriaPropiedad cp)
        {
            if(this.CategoriaPropiedad.Any(cl=>cl.IdPropiedad==cp.IdPropiedad && cl.IdCategoria==cp.IdCategoria))
                throw new CoreException(CoreException.PropiedadCategoriaReply);
           
              
              //  _iModelo.nuevaCategoriaPropiedad(cp);
                this.CategoriaPropiedad.Add(cp);
                cp.State = State.New;
        }
        public void guardarDireccion(Direccion dir,Elemento e)
        {
                      
                dir.IdElemento = e.Id;
             //   _iModelo.nuevoDireccion(dir);
                 __direcciones.Add(dir);
              
               
        }
        public void guardarCategoria(Categoria c)
        {
           
           //    _iModelo.nuevoCategoria(c);
                this.Categorias.Add(c);
                c.State = State.New;
        }
        public void guardarIdioma(Idioma i)
        {
      //      _iModelo.nuevoIdioma(i);
            this.Idiomas.Add(i);
            i.State = State.New;
        }

        /// <summary>
        /// Save all elements of the list.
        /// </summary>
        public void saveElements()
        {
           
           

            _iModelo.nuevoListaElemento( __Elmentos.Where(e => e.State == State.New).ToList());
            _iModelo.actualizarListaElementos(__Elmentos.Where(e => e.State == State.Modify).ToList());

            foreach (var i in Elementos) 
               i.State=State.Unchanged;
            
          
        }

        public void saveProperty()
        {
                _iModelo.nuevaListaPropiedad(__propiedades.Where(e => e.State == State.New).ToList());
                _iModelo.actualizarListaPropiedad(__propiedades.Where(e => e.State == State.Modify).ToList());
                foreach (var i in Propiedades)
                    i.State = State.Unchanged;
        }
        public void saveTP()
        {
           
                _iModelo.nuevaListaTraduccionPropiedad(__traducciones.Where(e => e.State == State.New).ToList());
                foreach (var i in Traducciones)
                    i.State = State.Unchanged;
        }
        public void saveVEP()
        {          
                _iModelo.nuevaListaValoresEP(__valores.Where(e => e.State == State.New).ToList());
                _iModelo.actualizarListaValoresEP(__valores.Where(e => e.State == State.Modify).ToList());
                foreach (var i in Valores)
                    i.State = State.Unchanged;
        }

        public void saveCategoryProperty()
        {
            _iModelo.nuevaListaCategoriaPropiedad(__categoriaPropiedad.Where(e => e.State == State.New).ToList());
            foreach (var i in CategoriaPropiedad)
                i.State = State.Unchanged;
        }
        public void saveCategory()
        {
            _iModelo.nuevoListaCategoria(__categoria.Where(e => e.State == State.New).ToList());
            _iModelo.actualizarListaCategoria(__categoria.Where(e => e.State == State.Modify).ToList());
            foreach (var i in Categorias)
                i.State = State.Unchanged;
        }

        public void saveIdioms()
        {
            foreach (var id in __idiomas.Where(e => e.State == State.New))
                _iModelo.nuevoIdioma(id);
            foreach (var i in Idiomas)
                i.State = State.Unchanged;

        }

        public void saveAll()
        {
         


            saveElements();
            saveCategory();
            saveCategoryProperty();
            saveIdioms();
            saveProperty();
            saveVEP();
            saveTP();
          
           
        }
        
        #endregion
        #region Consultas
        public Idioma devolverIdiomaPorCod(string cod)
        {

            Idioma idiomaVC = GestionDatos.Instance.Idiomas.FirstOrDefault(i => i.Cod.Equals(cod));
            if (idiomaVC == null)
                throw new CoreException(CoreException.CODNotFound);
            return idiomaVC;
        }

        /// <summary>
        /// Devuelve una lista de elementos que tengan una determinada categoria. Esto se utiliza para inicializar elementos de una capa superior.
        /// </summary>
        /// <param name="categoria"></param>
        /// <returns></returns>
        public List<Elemento> consultarLosElementosControladorPorCategoria(string categoria)
        {

            var elementos = (from ec in this.Elementos
                             where ec.Categorias[categoria] != null
                             select ec).ToList();


            return elementos;
        }

      
        
        #endregion
        #region LogicaID
        public int devolverProximoIdElemento()
        {
            if (this.Elementos.Count > 0)
                return (this.Elementos.Max().Id) + 1;
            return 1;
        }
        public int devolverProximoIdPropiedad()
        {
            if (this.Propiedades.Count > 0)
                return (this.Propiedades.Max().Id) + 1;
            return 1;
        }
        public int devolverProximoIdCategoria()
        {
            if (this.Categorias.Count > 0)
                return (this.Categorias.Max().Id) + 1;
            return 1;
        }
        public int devolverProximoIdCategoriaPropiedad()
        {
            if (this.CategoriaPropiedad.Count > 0)
                return (this.CategoriaPropiedad.Max().Id) + 1;
            return 1;
        }
        public int devolverProximoIdDireccion()
        {
            if (this.Direcciones.Count > 0)
                return (this.Direcciones.Max().Id) + 1;
            return 1;
        }
        public int devolverProximoIdTraduccionPropiedad()
        {
            if (this.Traducciones.Count > 0)
                return (this.Traducciones.Max().Id) + 1;
            return 1;
        }
        public int devolverProximoIdVEP() //TODO Actualizar la lista de valores al modificar/insertar
        {                                 
            
            if (this.Valores.Count > 0)
                return (this.Valores.Max().Id) + 1;
            return 1;
        }
        #endregion

    }
}
