using System;
using System;
using System.Collections.Generic;
using CoreLibrary.Datos;
using System.Linq;
using CoreLibrary;
using CoreLibrary.Controlador.Colecciones;
using System.Xml.Serialization;
namespace CoreLibrary.Datos.Modelo
{
	/// <summary>
	/// Clase que contiene las categorias.
	/// </summary>
	
	public class Categoria: ControladorBase
    {
        #region Constructor
        public Categoria() { }
        public Categoria(string categoria)
        {
            if (GestionDatos.Instance.Categorias.Any(c => c.Nombre.Equals(categoria)))
                throw new CoreException(CoreException.NameCategoryReply);
            this.Id = GestionDatos.Instance.devolverProximoIdCategoria();
            this.Nombre = categoria;
            this.Baja = false;

           GestionDatos.Instance.guardarCategoria(this);
            
        }
        #endregion
        #region Vars
        
        #endregion
        #region Propiedades
        [XmlAttribute]
        public string Nombre
        {
            get
            {
                return base.Nombre;
            }
            set
            {
                if (this.Nombre != null)
                  if (GestionDatos.Instance.Categorias.Any(c => c.Nombre.Equals(value) && c.Id != this.Id))
                    throw new CoreException(CoreException.NameCategoryReply);
                base.Nombre = value;
            }
        }

        /// <summary>
        /// Devuelve todas las propiedades asociadas a la instancia de esta categoria
        /// </summary>
        [XmlIgnore]
        public List<Propiedad> Propiedades
        {
            get
            {
                return (from p in GestionDatos.Instance.Propiedades
                        join cp in GestionDatos.Instance.CategoriaPropiedad on p.Id equals cp.IdPropiedad
                        where cp.IdCategoria == this.Id
                        select p).ToList(); 

            }
        }
        /// <summary>
        /// Devuelve la propiedad asociada a la categoria para el nombre pasado por indice. 
        /// </summary>
        /// <param name="nombre"></param>
        /// <returns></returns>
        [XmlIgnore]
        public Propiedad this[string nombre]
        {
            get
            {
                return this.Propiedades.FirstOrDefault(p => p.Nombre.Equals(nombre));
            }

        }
         [XmlIgnore]
        public List<Elemento> Elementos
        {
            get
            {
                return null;

            }

        }
        #endregion
        #region Metodos Publicos
        /// <summary>
        /// Aade la propiedad a la categoria actual.
        /// </summary>
        /// <param name="p"></param>
        public void aadirPropiedad(Propiedad p)
        {
             new CategoriaPropiedad(this, p);
            
        }
        /// <summary>
        /// Comprueba si tiene la propiedad pasada por parametro.
        /// </summary>
        /// <param name="nombre"></param>
        /// <returns></returns>
        public bool comprobarSiTienePropiedad(string nombre)
        {
            if (this[nombre] == null)
                return false;
            return true;
        }

        
        #endregion
    }
}

