using System;
using System.Collections.Generic;
using CoreLibrary.Controlador.Colecciones;
using System.Linq;
using System.Xml.Serialization;
namespace CoreLibrary.Datos.Modelo
{
    
    /// <summary>
    /// Clase que representa un elemento
    /// </summary>
    /// 
    public class Elemento : ControladorBase
    {
        /*
         * Metodo insertar nuevo elemento completado
         * Eliminar sobrecarga de modificarElemento
         * Contactar
         * 
         * */

        #region Constructor
        public Elemento() {
           
        }
        public Elemento(string nombre)
        {
            if(GestionDatos.Instance.Elementos.Any(e=>e.Nombre.Equals(nombre)))
                throw new CoreException(CoreException.NameElementoReply);
            this.Id = GestionDatos.Instance.devolverProximoIdElemento();
            this.Nombre=nombre;
            this.Baja = false;


            GestionDatos.Instance.guardarElemento(this);
           
        }
        #endregion

        #region Vars
        #endregion
        #region Propiedades publicas
       [XmlAttribute]
        public string Nombre
        {
            get
            {
                return base.Nombre;
            }
            ///El nuevo nombre no puede existir, en caso de que exista lanza una excepcion.
            set
            {
                //if(this.Nombre!=null)
                //    if (GestionDatos.Instance.Elementos.Any(e => e.Nombre.Equals(value) && e.Id != this.Id))
                //      throw new CoreException(CoreException.NameElementoReply);
                   
                       base.Nombre = value;
                
            }
        }
        [XmlIgnore]
        public CoreList<Categoria> Categorias 
        {
            get
            {

                CoreList<Categoria> categorias = new CoreList<Categoria>();
                foreach (ValoresEP vep in this.ValoresEP)
                {
                    foreach (Categoria cat in vep.Categorias)
                    {
                        if(!categorias.Contains(cat))
                            categorias.Add(cat);
                    }
                }

                return categorias;
            }

        }
          [XmlIgnore]
        public List<ValoresEP> ValoresEP
        {
            get
            {
              return (List<ValoresEP>)GestionDatos.Instance.TablaElementosValoresEP[this.Id];
            }
        }

       
        public ValoresEP this[string nombrePropiedad]
        {
            get
            {
                return this.ValoresEP.FirstOrDefault(vep => vep.Propiedad.Nombre.Equals(nombrePropiedad));
            }

        }
        
        #endregion
        #region Metodos
        /// <summary>
        /// Metodo que persiste los cambios en la base de datos (solo los de esta clase, no los de sus dependencias) y actualiza la lista 
        /// de elementos de la apliacin.
        /// </summary>
        public void guardarCambios()
        {
            GestionDatos.Instance.modificarElemento(this);        
        }
        public ValoresEP getValorPorNombre(string nombrePropiedad)
        {
            return this.ValoresEP.FirstOrDefault(vep => vep.Nombre.Equals(nombrePropiedad));
        }

        /// <summary>
        /// Metodo que devuelve para una categora pasada por parametro una lista de valoresEP sin repeticion de idPropiedad (USADA PARA CLASES QUE SE INSTANCIAN A TRAVES TBASE).
        /// Es getTodosLOsValoresEpDeUna... * n
        /// </summary>
        /// <param name="categoria"></param>
        /// <returns></returns>
        public List<ValoresEP> getUnicoValorEPDeUnaCategoria(String categoria)
        {
            List<ValoresEP> valoresEP = new List<ValoresEP>();
            foreach (var vep in getTodosLosValoresDeTodasLasCategorias(categoria))
                if (!valoresEP.Any(v=> v.IdPropiedad == vep.IdPropiedad))
                    valoresEP.Add(vep);
            
            return valoresEP;
        }
        /// <summary>
        /// Devuelve una lista de valoresEP 
        /// </summary>
        /// <param name="categoria"></param>
        /// <returns></returns>
        public List<ValoresEP> getTodosLosValoresDeTodasLasCategorias(string categoria)
        {
            return ValoresEP.Where(v => v.Categorias.FirstOrDefault(c=>c.Nombre.Equals(categoria))!=null).ToList();
        }
        #endregion
    }
}

