﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CoreLibrary.Datos;
using CoreLibrary.Datos.Modelo;
using CoreLibrary.Controlador.Colecciones;
using CoreLibrary.Literales;
using CoreLibrary.Controlador.Tipo;
namespace GuideLibrary
{
   public class Atributo:Propiedad
    {
        #region Constructor
        /// <summary>
        /// Crea una nueva instancia de atributos de un Elemento
        /// </summary>
        /// <param name="e">Elemento con el que se va a asociar</param>
        /// <param name="nombre">Nombre de la propiedad</param>
        /// <param name="valor">Valor que tendra al instanciarse, por defecto es true</param>
        /// <param name="cate">En el caso de que solo se quiera asignar una categoria.</param>
        /// <param name="categorias">Conjunto de categorias a las que pertenece</param>
        public Atributo(Elemento e, string nombre, bool valor = true, Categoria cate = null, CoreList<Categoria> categorias = null)
        {
            this._Elemento = e;
            //Se carga la propiedad, si no existe se crea.
            Propiedad propiedad = GestionDatos.Instance.Propiedades.FirstOrDefault(p => p.Nombre.Equals(nombre));
            if (propiedad == null)
                propiedad = new Propiedad(nombre, LTipoPrimitivo.Bool);

            //Se asignan los valores de la propiedad
            this.Nombre = propiedad.Nombre;
            this.Id = propiedad.Id;
            this.Baja = propiedad.Baja;
            this.TipoPrimitivo = propiedad.TipoPrimitivo;



            //Se asignan las categorias en caso de que se hayan pasado por parametro y no las tenga la propiedad.
            if (categorias == null && cate != null)
            {
                categorias = new CoreList<Categoria>();
                categorias.Add(cate);
            }

            if (categorias != null)
                foreach (Categoria cat in categorias)
                    if (!this.comprobarSiPertenceALaCategoria(cat.Nombre))
                        this.añadirCategoria(cat);

            //Se establece la relacion entre la propiedad y el Elemento.
            new TBoolean(_Elemento, propiedad, valor);

        }
        public Atributo()
        {

        }

        #endregion
        #region MetodosEstaticos

        public static Atributo cargar(Elemento e, string nombre)
        {
            Atributo ae = new Atributo();
            ae._Elemento = e;
            Propiedad propiedad = Propiedad.cargarPropiedad(e.getValorPorNombre(nombre));
            if (propiedad == null)
                return null;
            //Se asignan los valores de la propiedad
            ae.Nombre = propiedad.Nombre;
            ae.Id = propiedad.Id;
            ae.Baja = propiedad.Baja;
            ae.TipoPrimitivo = propiedad.TipoPrimitivo;

            return ae;
        }
        #endregion



        #region Vars
        private Elemento _Elemento;
        #endregion

        #region Propiedades

        public string Traduccion
        {
            get
            {
                if (this.Valor)
                    return this.devolverTraduccionPropiedad("es");

                return null;
            }

        }
        /// <summary>
        /// Al asignarse el valor se almacena automaticamente en la base de datos.
        /// </summary>
        public bool Valor
        {
            get
            {
                return new TBoolean(_Elemento, this).ValorBooleano;
            }
            set
            {
                new TBoolean(_Elemento, this).ValorBooleano = value;
            }

        }
        #endregion
    }
}
